local mod = MorgDKP2:NewModule("RaidTracker")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")
local media = LibStub("LibSharedMedia-3.0", true)
local BC = LibStub("LibBabble-Class-3.0"):GetLookupTable()


local mdkp = MorgDKP2
mod.modName = "RaidTracker"

--locals
local LL = LibStub("AceLocale-3.0"):NewLocale("RaidTracker", "enUS", true)
	
if LL then
	LL["Raid Tracker"] = true
	LL["Enable"] = true
	LL["Raid Tracker for viewing and editing current raids."] = true
	LL["Raids"] = true
	LL["Item DB"] = true
	LL["Aliases"] = true
	LL["DKP Changes"] = true
	LL["Attendees"] = true
end

local LL = LibStub("AceLocale-3.0"):GetLocale("RaidTracker")

local Player = UnitName("player")
local db
local fmt = string.format

local defaults = { 
	profile = {
		frames = {	
				raidtracker = {point = "CENTER", relpoint = "CENTER", x = 0, y = 0, width = 400, height = 400}
		},
	},
}

local options = {
		type = "group",
		handler = mod,
		name = LL["Raid Tracker"],
		desc = LL["Raid Tracker"],
		order = 400,
		args = {
			enable = {
				type = "toggle",
				name = LL["Enable"],
				desc = LL["Raid Tracker for viewing and editing current raids."],
				get = function() return mdkp.db.profile.modules.RaidTracker end,
				set = 	function(info, v) 
						mdkp.db.profile.modules.RaidTracker = v
						if v then 
							mod:Enable()
							mod:debug("Enabled Whisper")
						else
							mod:Disable()
							mdkp:debug("Disabled Whisper")
						end
					end,
				order = 5
			},
		},
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return nil
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("RaidTracker", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["RaidTracker"] = true
	
	self.scrolldata = nil
	self.font = media:Fetch("font", db.media.font)
	
end

function mod:OnDisable()
	db.moduleON["RaidTracker"] = nil
						
end

function mod:OpenFrame()
	if not mdkp.frames[name] then self:CreateRaidtrackerframe() end
	if db.raidnum == 0 then return end
	self:ShowRaid(db.raidnum)
end

function mod:CreateRaidtrackerframe()
	local name = "raidtracker"
	local background = media:Fetch("background", db.media.background)
	local border = media:Fetch("border", db.media.border)
	local font = media:Fetch("font", db.media.font)
	local fontsize = db.media.fontsize
	
	local minwidth = 8
	local title = "Raid Tracker"
	local ondragstart = function(f) f:StartMoving() end
	local ondragstop =  function(f)
   					f:StopMovingOrSizing()
   					local point, _, relativePoint, x, y = f:GetPoint()
					self.db.profile.frames[name].point = point
					self.db.profile.frames[name].relpoint = relativePoint
					self.db.profile.frames[name].x = x
					self.db.profile.frames[name].y = y
   				end
   	local onmouseup = function(f, button) if button == "RightButton" then mdkp.querytooltips[f.datacheck] = nil f:Hide() end end 
   	mdkp.frames[name] = self:Createframe(nil, nil, nil, nil, onmouseup, nil, name, nil, nil, title, ondragstart, ondragstop)
	
	local f = mdkp.frames[name]
	local framepos = self.db.profile.frames[name]
	local point, relpoint, x, y = framepos.point, framepos.relpoint, framepos.x, framepos.y
	f:ClearAllPoints()
	f:SetPoint(point, UIParent, relpoint, x, y)
	f:SetResizable(1)
	f:SetFrameStrata("DIALOG")
	
	f.close = CreateFrame("Button", nil, f)
   	f.close:SetScript("OnClick", function(f) f:GetParent():Hide() end)
   	f.close:SetWidth(20)
   	f.close:SetHeight(20)
   	f.close:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
   	f.close:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
   	f.close:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
   	f.close:ClearAllPoints()
   	f.close:SetPoint("TOPRIGHT", f, "TOPRIGHT")
   	f.close:SetHitRectInsets(5, 5, 5, 5)
   	f.close:Show()
   	
   	f.size = CreateFrame("Button", nil, f)
   	f.size:SetScript("OnMouseDown", function(f) f:GetParent():StartSizing() end)
   	f.size:SetScript("OnMouseUp", function(f)
   					local p = f:GetParent()
   					p:StopMovingOrSizing()
   					local width, height = p:GetWidth(), p:GetHeight()
					self.db.profile.frames[name].width = width
					self.db.profile.frames[name].height = height
					self:AdjustFrames(width, height)
   				end)
   	f.size:SetWidth(10)
   	f.size:SetHeight(10)
   	f.size:SetNormalTexture("Interface\\Buttons\\UI-AutoCastableOverlay")
   	local texture = f.size:GetNormalTexture()
   	texture:SetTexCoord(.629, .759, .615, .770)
   	texture:SetVertexColor(unpack(db.media.borderC))
   	f.size:SetPushedTexture("Interface\\Buttons\\UI-AutoCastableOverlay")
   	local texture = f.size:GetPushedTexture()
   	texture:SetTexCoord(.629, .759, .615, .770)
   	texture:SetVertexColor(1,0,0,1)
   	f.size:SetHighlightTexture("Interface\\Buttons\\UI-AutoCastableOverlay")
   	local texture = f.size:GetHighlightTexture()
   	texture:SetTexCoord(.629, .759, .615, .770)
   	texture:SetVertexColor(1,1,1,1)
   	f.size:ClearAllPoints()
   	f.size:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -2, 2)
   	f.size:Show()
   	
   	f.buttons = {}
   	f.buttons[1] = CreateFrame("Button", nil, f)
   	f.buttons[1]:SetScript("OnClick", function(f) self:ShowRaidFrame(f:GetID()) end)
   	f.buttons[1]:SetNormalTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Up")
   	local texture = f.buttons[1]:GetNormalTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	f.buttons[1]:SetHighlightTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Glow")
   	local texture = f.buttons[1]:GetHighlightTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	f.buttons[1]:ClearAllPoints()
   	f.buttons[1]:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT", 4, 4)
   	f.buttons[1]:SetHitRectInsets(5, 5, 5, 5)
   	f.buttons[1]:SetID(1)
   	f.buttons[1]:Show()
   	
   	f.buttons[1].text = f.buttons[1]:CreateFontString()
   	f.buttons[1].text:SetFont(font, fontsize)
   	f.buttons[1].text:ClearAllPoints()
	f.buttons[1].text:SetPoint("CENTER", f.buttons[1], "CENTER",0,0)
	f.buttons[1].text:SetText(LL["Raids"])
	local width = f.buttons[1].text:GetStringWidth() + 15
	f.buttons[1].text:SetWidth(width)
	f.buttons[1]:SetWidth(width)
	f.buttons[1]:SetHeight(25)
   	minwidth = minwidth + width
   	
   	f.buttons[2] = CreateFrame("Button", nil, f)
   	f.buttons[2]:SetScript("OnClick", function(f) self:ShowRaidFrame(f:GetID()) end)
   	f.buttons[2]:SetNormalTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Up")
   	local texture = f.buttons[2]:GetNormalTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	f.buttons[2]:SetHighlightTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Glow")
   	local texture = f.buttons[2]:GetHighlightTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	f.buttons[2]:ClearAllPoints()
   	f.buttons[2]:SetPoint("LEFT", f.buttons[1], "RIGHT", 0, 0)
   	f.buttons[2]:SetID(2)
   	f.buttons[2]:Show()
   	
   	f.buttons[2].text = f.buttons[2]:CreateFontString()
   	f.buttons[2].text:SetFont(font, fontsize)
   	f.buttons[2].text:ClearAllPoints()
	f.buttons[2].text:SetPoint("CENTER", f.buttons[2], "CENTER",0,0)
	f.buttons[2].text:SetText(LL["Item DB"])
	local width = f.buttons[2].text:GetStringWidth() + 15
	f.buttons[2].text:SetWidth(width)
	f.buttons[2]:SetWidth(width)
	f.buttons[2]:SetHeight(25)
   	minwidth = minwidth + width
   	
	f.buttons[3] = CreateFrame("Button", nil, f)
   	f.buttons[3]:SetScript("OnClick", function(f) self:ShowRaidFrame(f:GetID()) end)
   	f.buttons[3]:SetNormalTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Up")
   	local texture = f.buttons[3]:GetNormalTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	f.buttons[3]:SetHighlightTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Glow")
   	local texture = f.buttons[3]:GetHighlightTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	f.buttons[3]:ClearAllPoints()
   	f.buttons[3]:SetPoint("LEFT", f.buttons[2], "RIGHT", 0, 0)
   	f.buttons[3]:SetHitRectInsets(5, 5, 5, 5)
   	f.buttons[3]:SetID(3)
   	f.buttons[3]:Show()
   	
   	f.buttons[3].text = f.buttons[3]:CreateFontString()
   	f.buttons[3].text:SetFont(font, fontsize)
   	f.buttons[3].text:ClearAllPoints()
	f.buttons[3].text:SetPoint("CENTER", f.buttons[3], "CENTER",0,0)
	f.buttons[3].text:SetText(LL["Aliases"])
	local width = f.buttons[3].text:GetStringWidth() + 15
	f.buttons[3].text:SetWidth(width)
	f.buttons[3]:SetWidth(width)
	f.buttons[3]:SetHeight(25)
   	minwidth = minwidth + width
   	
	f.buttons[4] = CreateFrame("Button", nil, f)
   	f.buttons[4]:SetScript("OnClick", function(f) self:ShowRaidFrame(f:GetID()) end)
   	f.buttons[4]:SetNormalTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Up")
   	local texture = f.buttons[4]:GetNormalTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	f.buttons[4]:SetHighlightTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Glow")
   	local texture = f.buttons[4]:GetHighlightTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	f.buttons[4]:ClearAllPoints()
   	f.buttons[4]:SetPoint("LEFT", f.buttons[3], "RIGHT", 0, 0)
   	f.buttons[4]:SetHitRectInsets(5, 5, 5, 5)
   	f.buttons[4]:SetID(4)
   	f.buttons[4]:Show()
   	
   	f.buttons[4].text = f.buttons[4]:CreateFontString()
   	f.buttons[4].text:SetFont(font, fontsize)
   	f.buttons[4].text:ClearAllPoints()
	f.buttons[4].text:SetPoint("CENTER", f.buttons[4], "CENTER",0,0)
	f.buttons[4].text:SetText(LL["DKP Changes"])
	local width = f.buttons[4].text:GetStringWidth() + 15
	f.buttons[4].text:SetWidth(width)
	f.buttons[4]:SetWidth(width)
	f.buttons[4]:SetHeight(25)
   	minwidth = minwidth + width
   	
   	f:SetMinResize(minwidth, 100)
   	local width, height = self.db.profile.frames[name].width, self.db.profile.frames[name].height					
   	if width < minwidth then width = minwidth end
   	f:SetWidth(width)
   	f:SetHeight(height)
   	
   	--raids frame
   	f.frame = {}
   	f.frame[1] = CreateFrame("frame", nil, f)
	local rf = f.frame[1]
	
	rf:ClearAllPoints()
   	rf:SetBackdrop({ bgFile = background, tile = false, tileSize = 16, edgeFile = border, edgeSize = 8, insets = { left = 6, right = 6, top = 6, bottom = 6 } })
	rf:SetBackdropColor(unpack(db.media.backdropC))
	rf:SetBackdropBorderColor(0,0,0,0)
	rf:SetPoint("CENTER", f, "CENTER")
	rf:SetHeight(height - 30)
	rf:SetWidth(width - 20)
	rf:Show()
	
	rf.RAID = CreateFrame("Button", nil, rf)
   	rf.RAID:SetScript("OnClick", function(f) self:DewthruMe(f, "raid") end)
   	rf.RAID:SetNormalTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Up")
   	local texture = rf.RAID:GetNormalTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	rf.RAID:SetHighlightTexture("Interface\\GLUES\\COMMON\\Glues-BigButton-Glow")
   	local texture = rf.RAID:GetHighlightTexture()
   	texture:SetTexCoord(.168, .818, .239, .753)
   	rf.RAID:ClearAllPoints()
   	rf.RAID:SetPoint("TOP", rf, "TOP", 0, -3)
   	rf.RAID:Show()
   	
   	rf.RAID.text = rf.RAID:CreateFontString()
   	rf.RAID.text:SetFont(font, fontsize)
   	rf.RAID.text:ClearAllPoints()
	rf.RAID.text:SetPoint("CENTER", rf.RAID, "CENTER")
	rf.RAID.text:SetText("Raid1")
	local buttonwidth = rf.RAID.text:GetStringWidth() + 15
	rf.RAID.text:SetWidth(buttonwidth)
	rf.RAID:SetWidth(buttonwidth)
	rf.RAID:SetHeight(20)
   	
   	rf.scroll = self:CreateScrollFrame(rf, background, border)
   	rf.scroll:SetPoint("TOPLEFT", rf, "TOPLEFT", 10, -25)
   	
	--items frame
   	f.frame[2] = CreateFrame("frame", nil, f)
	local rf = f.frame[2]
	
	rf:ClearAllPoints()
   	rf:SetBackdrop({ bgFile = background, tile = false, tileSize = 16, edgeFile = border, edgeSize = 8, insets = { left = 6, right = 6, top = 6, bottom = 6 } })
	rf:SetBackdropColor(unpack(db.media.backdropC))
	rf:SetBackdropBorderColor(unpack(db.media.borderC))
	rf:SetPoint("CENTER", f, "CENTER")
	rf:SetHeight(height - 30)
	rf:SetWidth(width - 20)
	rf:Hide()
		
	rf.title = rf:CreateFontString()
   	rf.title:SetPoint("TOP", rf, "TOP", 0, -10)
   	rf.title:SetFont(font, 14)
   	rf.title:SetText("Item DB")
   	rf.title:SetTextColor(unpack(db.media.borderC))
   	rf.title:Show()
   	
	--aliases frame
   	f.frame[3] = CreateFrame("frame", nil, f)
	local rf = f.frame[3]
	
	rf:ClearAllPoints()
   	rf:SetBackdrop({ bgFile = background, tile = false, tileSize = 16, edgeFile = border, edgeSize = 8, insets = { left = 6, right = 6, top = 6, bottom = 6 } })
	rf:SetBackdropColor(unpack(db.media.backdropC))
	rf:SetBackdropBorderColor(unpack(db.media.borderC))
	rf:SetPoint("CENTER", f, "CENTER")
	rf:SetHeight(height - 30)
	rf:SetWidth(width - 20)
	rf:Hide()
		
	rf.title = rf:CreateFontString()
   	rf.title:SetPoint("TOP", rf, "TOP", 0, -10)
   	rf.title:SetFont(font, 14)
   	rf.title:SetText("Aliases")
   	rf.title:SetTextColor(unpack(db.media.borderC))
   	rf.title:Show()
   	
   	--dkp changes frame
   	f.frame[4] = CreateFrame("frame", nil, f)
	local rf = f.frame[4]
	
	rf:ClearAllPoints()
   	rf:SetBackdrop({ bgFile = background, tile = false, tileSize = 16, edgeFile = border, edgeSize = 8, insets = { left = 6, right = 6, top = 6, bottom = 6 } })
	rf:SetBackdropColor(unpack(db.media.backdropC))
	rf:SetBackdropBorderColor(unpack(db.media.borderC))
	rf:SetPoint("CENTER", f, "CENTER")
	rf:SetHeight(height - 30)
	rf:SetWidth(width - 20)
	rf:Hide()
		
	rf.title = rf:CreateFontString()
   	rf.title:SetPoint("TOP", rf, "TOP", 0, -10)
   	rf.title:SetFont(font, 14)
   	rf.title:SetText("DKP Changes")
   	rf.title:SetTextColor(unpack(db.media.borderC))
   	rf.title:Show()
   	
   	mdkp.frames[name]:Show()
   	
end

function mod:ShowRaidFrame(ID)
	mdkp.frames.raidtracker.frame[ID]:Show()
	mdkp.frames.raidtracker.buttons[ID]:LockHighlight()
	for num = 1,4 do
		if num ~= ID then
			mdkp.frames.raidtracker.buttons[num]:UnlockHighlight()
			mdkp.frames.raidtracker.frame[num]:Hide()
		end
	end
end

function mod:AdjustFrames(width, height)
	for num = 1,4 do
		mdkp.frames.raidtracker.frame[num]:SetHeight(height - 30)
		mdkp.frames.raidtracker.frame[num]:SetWidth(width - 20)
	end
	self:UpdateScrollFrame(mdkp.frames.raidtracker.frame[1], "raids")
end

function mod:DewthruMe(frame, type)
	local validate = {}
	if type == "raid" then
		if db.raidnum == 0 then return end
		for num = 1, db.raidnum do
			tinsert(validate, db.raidlog[num].zone or num)
		end
	end
	self:Opendewdrop(frame, validate, "raidtracker", type)
end

function mod:HandleDewAction(type, value, id)
	self:debug("activating "..type.." to "..value)
	if type == "raid" then self:ShowRaid(id)
	end
end

function mod:ShowRaid(raidnum)
	local eventname = db.raidlog[raidnum].zone
	local rf = mdkp.frames.raidtracker.frame[1]
	rf.RAID.text:SetText(eventname)
	local buttonwidth = rf.RAID.text:GetStringWidth() + 15
	rf.RAID.text:SetWidth(buttonwidth)
	rf.RAID:SetWidth(buttonwidth)
	
	--add raids and items
	local numevents = #db.raidlog[raidnum].bosskills
	self.scrolldata = { }
	if numevents > 0 then 
		for event, eventdata in ipairs(db.raidlog[raidnum].bosskills) do
			local note = eventdata.note
			tinsert(self.scrolldata, { name = eventdata.name, value = eventdata.value, time = eventdata.time, attendees = eventdata.attendees, note = note, trash = eventdata.trash, index = event})
			if eventdata.trash ~= 1 then
				--add items
				if db.raidlog[raidnum].loot then
					for index, itemdata in ipairs(db.raidlog[raidnum].loot) do
						if eventdata.note == itemdata.Boss or (itemdata.Boss == L["Trash mob"] and eventdata.trash == 2) then
							tinsert(self.scrolldata, {link = itemdata.ItemLink, name = itemdata.ItemName, winner = itemdata.Player, value = itemdata.Costs, quality = mdkp.Quality[itemdata.Quality+1], icon = itemdata.Icon, trash = nil, index = index})
						end
					end
   				end
   			end
   		end
   		if db.raidlog[raidnum].loot then
			local added = nil
			for index, itemdata in ipairs(db.raidlog[raidnum].loot) do
				if itemdata.Boss == L["Bank"] then
					if not added then
						tinsert(self.scrolldata, { name = L["Bank"], value = L["Bank"], time = nil, attendees = nil, note = note, trash = -1, index = nil})
						added = true
					end
					tinsert(self.scrolldata, {link = itemdata.ItemLink, name = itemdata.ItemName, winner = itemdata.Player, value = itemdata.Costs, quality = mdkp.Quality[itemdata.Quality+1], icon = itemdata.Icon, trash = nil, index = index})
				end
			end
		end	
   	end
   	self:UpdateScrollFrame(rf, "raids")
   	
end	

function mod:UpdateScrollFrame(f, framename)
	local sf = f.scroll
	local fontallowance = db.media.fontsize * 3 - 25
	if fontallowance < 0 then fontallowance = 0 end
	local frameheight = 25 + fontallowance
	local totalheight = self.db.profile.frames["raidtracker"].height
	local maxitems = floor(((totalheight - 125)/frameheight) - .5)
	local attach, relpoint, x, y
	self:debug(frameheight.." "..totalheight.."="..maxitems)
	
	--clear old
	for i = 1, sf.maxitems do
		sf.buttons[i]:Hide()
	end
	sf.maxitems = maxitems
	
	--populate frame
	local sr, sg, sb = unpack(db.media.borderC)
	local numitems = #self.scrolldata
	if numitems < maxitems then 
		maxitems = numitems 
		sr, sg, sb = 0, 0, 0
	end
	for id = 1, numitems do
		local data = self.scrolldata[id]
		if not sf.buttons[id] then 
			sf.buttons[id] = self:CreateLootbutton(id, fontallowance, sf) 
			if id == 1 then 
				attach = sf
				relpoint = "TOPLEFT"
				x = 5
				y = -5
			else 
				attach = sf.buttons[id-1]
				relpoint = "BOTTOMLEFT"
				x = 0
				y = 0
			end
			sf.buttons[id]:SetPoint("TOPLEFT", attach, relpoint, x , y)
		end
		if id == maxitems then break end
	end
	
	--position frame
	sf:SetHeight((30 + fontallowance) * maxitems + 10)
	sf:SetWidth(190)
	sf:SetBackdropBorderColor(sr, sg, sb, 1)
	sf:Show()
	
	--update frame
	sf:SetScript("OnMouseWheel", function(f, direction) self:RaidTrackerScrollframe_Update(f, numitems, maxitems, direction, framename) end)
	self:RaidTrackerScrollframe_Update(sf, numitems, maxitems, nil, framename)
end

function mod:RaidTrackerScrollframe_Update(f, numitems, maxitems, direction, framename)
	local line
	local lineplusoffset
	local change = 1
	local offset = floor(f:GetVerticalScroll() + 0.5)
	local x = 0
	local r,g,b,a = 1,1,1,1
	local r2,g2,b2,a2
	
	if IsControlKeyDown() then
		if direction == 1 then offset = 0
		elseif direction == -1 then offset = numitems - maxitems end
	else
		if IsShiftKeyDown() then change = 10 end
		if direction == -1 then offset = offset + change
		elseif direction == 1 then offset = offset - change end
	end
	if offset < 0 then offset = 0 
	elseif offset > (numitems - maxitems) then offset = numitems - maxitems end
	f:SetVerticalScroll(offset)
	for line = 1, maxitems do
		lineplusoffset = line + offset
		if lineplusoffset <= numitems then
			local value = self.scrolldata[lineplusoffset]
			if value.trash then
				--not an item
				f.buttons[line].name:SetText(value.name)
				f.buttons[line].name:SetTextColor(r,g,b,a)
				if value.trash ~= -1 then
					f.buttons[line].icon:SetNormalTexture("Interface\\ICONS\\Achievement_Quests_Completed_08")
					f.buttons[line].note:SetText(value.note)
					f.buttons[line].note:SetTextColor(r,g,b,a)
					f.buttons[line].note:Show()
					f.buttons[line].winner:SetText(value.value)
					f.buttons[line].winner:SetTextColor(0,1,0,1)
					f.buttons[line].dkp:Hide()
					f.buttons[line].winner:Show()
				else 
					f.buttons[line].icon:SetNormalTexture("Interface\\ICONS\\INV_Enchant_ShardBrilliantLarge")
					f.buttons[line].dkp:Hide()
					f.buttons[line].winner:Hide()
					f.buttons[line].note:Hide()
				end
				x = 0
				r2,g2,b2,a2 = 1,1,0,.5
			else
				--is an item
				f.buttons[line].icon:SetNormalTexture(value.icon)
				f.buttons[line].name:SetText(value.name)
				f.buttons[line].name:SetTextColor(value.quality.r, value.quality.g, value.quality.b)
				f.buttons[line].dkp:SetText(value.value)
				f.buttons[line].dkp:SetTextColor(1,0,0)
				f.buttons[line].winner:SetText(value.winner)
				f.buttons[line].winner:SetTextColor(1,1,1)
				f.buttons[line].dkp:Show()
				f.buttons[line].winner:Show()
				f.buttons[line].note:Hide()
				x = 20
				r2,g2,b2,a2 = value.quality.r, value.quality.g, value.quality.b, 1
			end
			--f.buttons[line]:SetPoint(
			f.buttons[line]:SetBackdropBorderColor(r2,g2,b2,a2)
			f.buttons[line]:SetID(lineplusoffset)
			f.buttons[line]:Show()
		else
			if f.buttons[line] then f.buttons[line]:Hide() end
		end
	end
end

function mod:CreateScrollFrame(parent, background, border)
	local f = CreateFrame("ScrollFrame", nil, parent)
   	f:EnableMouse(1)
   	f:ClearAllPoints()
   	f:EnableMouseWheel(1)
   	f:SetBackdrop({ bgFile = background, tile = false, tileSize = 16, edgeFile = border, edgeSize = 8, insets = { left = 3, right = 0, top = 3, bottom = 3 } })
	f:SetBackdropColor(0,0,0,1)
	f:SetBackdropBorderColor(0, 0, 0, 1)
	f.buttons = {}
	f.maxitems = 0
	return f
end

function mod:CreateLootbutton(id, fontallowance, frame)
	self:debug("Adding button...")
	local f = CreateFrame("frame", nil, frame)
	local textwidth = 180 - 25 - fontallowance - 7
	
	f:SetID(id)
	f:SetFrameStrata("DIALOG")
   	f:EnableMouse(1)
   	f:ClearAllPoints()
   	f:SetBackdrop({ bgFile = media:Fetch("background", "Solid"), tile = false, tileSize = 16, edgeFile = media:Fetch("border", "Blizzard Tooltip"), edgeSize = 8, insets = { left = 3, right = 3, top = 3, bottom = 3 } })
	f:SetBackdropColor(0,0,0,1)
	f:SetBackdropBorderColor(1, 1, 1, 1)
	f:SetScript("OnMouseUp", function(f, button) mod:ClicktoEdit(f, f:GetID(), button) end)
	f:SetHeight(30 + fontallowance)
	f:SetWidth(180)
	
	f.icon = CreateFrame("Button", nil, f)
	f.icon:SetWidth(25 + fontallowance)
	f.icon:SetHeight(25 + fontallowance)
	f.icon:ClearAllPoints()
	f.icon:SetPoint("TOPLEFT", f, "TOPLEFT", 3, -3)
	f.icon:SetScript("OnEnter", function(f)
						local index = f:GetParent():GetID()
						if self.scrolldata[index].link then 
							GameTooltip:SetOwner(f, "ANCHOR_RIGHT")
							GameTooltip:SetHyperlink(self.scrolldata[index].link)
							GameTooltip:Show()
						elseif self.scrolldata[index].attendees then self:DoAttendeesTip(f, self.scrolldata[index].attendees)
						end
					end)
	f.icon:SetScript("OnLeave", function() GameTooltip:Hide() end)
	f.icon:SetScript("OnMouseUp", function(f, button) mod:ClicktoEdit(f, f:GetParent():GetID(), button) end)
	f.icon:Show()
	
   	f.name = f:CreateFontString()
   	f.name:SetPoint("TOPLEFT", f.icon, "TOPRIGHT", 5, 2)
   	f.name:SetWidth(textwidth)
   	f.name:SetJustifyH("LEFT")
   	f.name:SetFont(self.font, db.media.fontsize)
   	f.name:Show()
   	
	f.note = f:CreateFontString()
   	f.note:SetPoint("LEFT", f.icon, "RIGHT", 5, 0)
   	f.note:SetWidth(textwidth)
   	f.note:SetJustifyH("LEFT")
   	f.note:SetFont(self.font, db.media.fontsize)
   	f.note:Show()
   	
   	f.dkp = f:CreateFontString()
   	f.dkp:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -5, 2)
   	f.dkp:SetWidth(textwidth)
   	f.dkp:SetJustifyH("RIGHT")
   	f.dkp:SetFont(self.font, db.media.fontsize)
   	f.dkp:Show()
   	
   	f.winner = f:CreateFontString()
   	f.winner:SetPoint("BOTTOMLEFT", f.icon, "BOTTOMRIGHT", 5, 0)
   	f.winner:SetWidth(textwidth)
   	f.winner:SetJustifyH("LEFT")
   	f.winner:SetTextColor(1, 1, 1)
   	f.winner:SetFont(self.font, db.media.fontsize)
   	f.winner:Show()
   	
   	return f
end

function mod:DoAttendeesTip(frame, attendees)
	GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
	GameTooltip:AddLine(LL["Attendees"])
	for _, name in ipairs(attendees) do
  		local colorname = self:GetClasscolors(name)
		GameTooltip:AddLine(name, colorname.r, colorname.g, colorname.b)
  	end
  	GameTooltip:Show()
end

function mod:ClicktoEdit(frame, ID, button)
	self:debug("click.."..ID)
end